#include "movieCollection.h"

Node* createNode(MovieRecord newData) // helper function
{
    Node* pMem = malloc(sizeof(Node)); // request allocating space for Node

    if (pMem != NULL)
    {
        // let's intialize the Node
        pMem->pLink = NULL;
        // pMem->data = newData; // struct assignment
        pMem->data.year = newData.year;
        strcpy(pMem->data.title, newData.title);
    }

    return pMem;
}

int insertAtFront(Node** pList, MovieRecord newData)
{
    Node* pMem = malloc(sizeof(Node)); // request allocating space for Node
    int success = 0; // failed to allocate space on the heap

    if (pMem != NULL)
    {
        // we allocated space for a Node successfully
        success = 1;

        // let's intialize the Node
        pMem->pLink = NULL;
       // pMem->data = newData; // struct assignment
        pMem->data.year = newData.year;
        strcpy(pMem->data.title, newData.title);

        // we init the new Node
        // empty list?
        if ((*pList) == NULL)
        {
            // it is empty list
            *pList = pMem;
        }
        else
        {
            // it is not empty
            pMem->pLink = *pList;
            *pList = pMem;
        }
    }

    return success;
}

// dictionary ordering (A - Z)
int insertInOrder(Node** pList, MovieRecord newData)
{
    Node* pMem = createNode(newData), *pCur = *pList, *pPrev = NULL;
    int success = 0;

    if (pMem != NULL)
    {
        // allocated space successfully for a Node
        success = 1;

        // if pCur is NULL, then either the list is empty or we're inserting
        // at the end of the list
        while (pCur != NULL && strcmp(newData.title, pCur->data.title) > 0)
        {
            pPrev = pCur;
            pCur = pCur->pLink;
        }

        // are we inserting at the front or an empty list?
        if (pPrev == NULL)
        {
            // yes, we're either inserting at the front or an empty list
            pMem->pLink = pCur; // *pList could be used instead of pCur as well
            *pList = pMem; // assign the head ptr to the new Node
        }
        else
        {
            // we're inserting somewhere in the middle or at the end
            pMem->pLink = pCur; // if we're inserting at the end, then pCur is NULL
            pPrev->pLink = pMem;
        }
    }

    return success;
}

// precondition: list must not be empty
MovieRecord deleteFront(Node** pList)
{
    Node* pTemp = *pList;
    MovieRecord rec = pTemp->data;

    // update the head ptr (pHead)
    *pList = pTemp->pLink;
    free(pTemp);

    return rec;
}

// precondition: list must not be empty
int deleteTitle(Node** pList, char* title)
{
    Node* pPrev = NULL, * pCur = *pList;
    int success = 0;

    // we need to search the list for the matching title to remove
    // from our collection. if the movie title doesn't exist
    // in our collection, then pCur will become the NULL ptr
    while (pCur != NULL && strcmp(title, pCur->data.title) != 0)
    {
        pPrev = pCur;
        pCur = pCur->pLink;
    }

    if (pCur != NULL) // did we find the title?
    {
        // yes, we found the title in our collection
        success = 1;

        // are we removing the first movie in the collection?
        if (pPrev == NULL)
        {
            // yes, we're removing the first movie 
            *pList = pCur->pLink; // update the head ptr
        }
        else
        {
            // we're removing a title somewhere in the middle or at the end of list
            pPrev->pLink = pCur->pLink;
        }

        // free up the space for the Node that stored the title that we removed from the list
        free(pCur);
    }

    return success;
}

void printList(Node* pList)
{
    printf("--> ");

    while (pList != NULL)
    {
        // there's another Node - print the data
        printf("%s, %d--> ", pList->data.title, pList->data.year);
        pList = pList->pLink;
    }

    putchar('\n');
}